from __future__ import print_function
from builtins import str
import os
import unittest
import shutil
import unittest
import yaml
from astrocalc.utKit import utKit
from fundamentals import tools
from os.path import expanduser
home = expanduser("~")

packageDirectory = utKit("").get_project_root()
settingsFile = packageDirectory + "/test_settings.yaml"
su = tools(
    arguments={"settingsFile": settingsFile},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName=None,
    defaultSettingsFile=False
)
arguments, settings, log, dbConn = su.setup()

# SETUP PATHS TO COMMON DIRECTORIES FOR TEST DATA
moduleDirectory = os.path.dirname(__file__)
pathToInputDir = moduleDirectory + "/input/"
pathToOutputDir = moduleDirectory + "/output/"

try:
    shutil.rmtree(pathToOutputDir)
except:
    pass
# COPY INPUT TO OUTPUT DIR
shutil.copytree(pathToInputDir, pathToOutputDir)

# Recursively create missing directories
if not os.path.exists(pathToOutputDir):
    os.makedirs(pathToOutputDir)


class test_unit_conversion(unittest.TestCase):

    def test_unit_conversion_sexe_to_deg_function(self):
        kwargs = {}
        kwargs["log"] = log
        kwargs["settings"] = settings
        # xt-kwarg_key_and_value

        # DEC FIRST
        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        dec = converter.dec_sexegesimal_to_decimal(
            dec="-23:45:21.23232"
        )
        print(dec)

        dec = converter.dec_sexegesimal_to_decimal(
            dec="+1:58:05.45341"
        )
        print(dec)

        dec = converter.dec_sexegesimal_to_decimal(
            dec="+01:58:5.45341"
        )
        print(dec)

        dec = converter.dec_sexegesimal_to_decimal(
            dec="01:58:05"
        )
        print(dec)

        dec = converter.dec_sexegesimal_to_decimal(
            dec="12.3234234234"
        )
        print(dec)

        dec = converter.dec_sexegesimal_to_decimal(
            dec="-34.3234234234"
        )
        print(dec)

        # TEST RA NOW
        ra = converter.ra_sexegesimal_to_decimal(
            ra="23:45:21.23232"
        )
        print(ra)

        # TEST RA NOW
        ra = converter.ra_sexegesimal_to_decimal(
            ra="23h45m21.23232s"
        )
        print(ra)

        # TEST RA NOW
        ra = converter.ra_sexegesimal_to_decimal(
            ra="23 45 21.23232"
        )
        print(ra)

        # TEST RA NOW
        ra = converter.ra_sexegesimal_to_decimal(
            ra="2 04 21.23232"
        )
        print(ra)

        # TEST RA NOW
        ra = converter.ra_sexegesimal_to_decimal(
            ra="04:45  21"
        )
        print(ra)

        ra = converter.ra_sexegesimal_to_decimal(
            ra="12.3234234234"
        )
        print(ra)

        ra = converter.ra_sexegesimal_to_decimal(
            ra="334.3234234234"
        )
        print(ra)

    def test_unit_conversion_deg_to_sex_functions(self):
        kwargs = {}
        kwargs["log"] = log
        kwargs["settings"] = settings
        # xt-kwarg_key_and_value

        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        ra = converter.ra_decimal_to_sexegesimal(
            ra="-23.454676456",
            delimiter="/"
        )
        print(ra)

        ra = converter.ra_decimal_to_sexegesimal(
            ra="63.454676456"
        )
        print(ra)

        ra = converter.ra_decimal_to_sexegesimal(
            ra="0.454676456"
        )
        print(ra)

        ra = converter.ra_decimal_to_sexegesimal(
            ra="345.454"
        )
        print(ra)

        ra = converter.ra_decimal_to_sexegesimal(
            ra="345"
        )
        print(ra)

        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        dec = converter.dec_decimal_to_sexegesimal(
            dec="-23.454676456",
            delimiter="/"
        )
        print(dec)

        dec = converter.dec_decimal_to_sexegesimal(
            dec="-83.454676456",
            delimiter=":"
        )
        print(dec)

        dec = converter.dec_decimal_to_sexegesimal(
            dec="-3.454676456",
            delimiter=":"
        )
        print(dec)

        dec = converter.dec_decimal_to_sexegesimal(
            dec="50",
            delimiter=":"
        )
        print(dec)

    def test_unit_conversion_ra_dec_to_cartesian_functions(self):

        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        x, y, z = converter.ra_dec_to_cartesian(
            ra=23.454676456,
            dec=-3.454676456
        )
        print(x, y, z)

        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        x, y, z = converter.ra_dec_to_cartesian(
            ra="23 45 21.23232",
            dec=-3.454676456
        )
        print(x, y, z)

        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        cartesians = converter.ra_dec_to_cartesian(
            ra="23 45 21.23232",
            dec="+01:58:5.45341"
        )
        print(cartesians)



from __future__ import division
from . import cl_utils
from . import docstring_test
from past.utils import old_div

def luminosity_to_flux(lumErg_S, dist_Mpc):
    """
    *Convert luminosity to a flux*

    **Key Arguments**

    - ``lumErg_S`` -- luminosity in ergs/sec
    - ``dist_Mpc`` -- distance in Mpc
    

    **Return**

    - ``fluxErg_cm2_S`` -- flux in ergs/cm2/s
    
    """
    ## STANDARD LIB ##
    ## THIRD PARTY ##
    import numpy as np
    import math
    ## LOCAL APPLICATION ##

    ################ > VARIABLE SETTINGS ######

    ################ >ACTION(S) ################
    # Convert the distance to cm
    distCm = dist_Mpc * MPC_2_CMS
    fluxErg_cm2_S = old_div(lumErg_S, (4 * np.pi * distCm ** 2))

    return fluxErg_cm2_S

##########################################################################
# PRIVATE (HELPER) FUNCTIONS                                                                #
##########################################################################
if __name__ == '__main__':
    main()



"""
*Calculations and conversions based on sky-coordinates*
"""
from __future__ import absolute_import

from .unit_conversion import unit_conversion
from .separations import separations
from .translate import translate
from .coordinates_to_array import coordinates_to_array



#!/usr/bin/env python
# encoding: utf-8
"""
Documentation for astrocalc can be found here: http://astrocalc.readthedocs.org

Usage:
    astrocalc [-c] coordflip <ra> <dec>
    astrocalc sep <ra1> <dec1> <ra2> <dec2>
    astrocalc timeflip <datetime>
    astrocalc trans <ra> <dec> <north> <east>
    astrocalc now mjd
    astrocalc dist (-z | -m) <distVal> [--hc=hVal --wm=OmegaMatter --wv=OmegaVacuum]

Commands:
    coordflip             flip coordinates between decimal degrees and sexegesimal and vice-versa
    sep                   calculate the separation between two locations in the sky.
    timeflip              flip time between UT and MJD.
    trans                 translate a location across the sky (north and east in arcsec)
    now                   report current time in various formats
    dist                  convert distance between mpc to z

Variables:
    ra, ra1, ra2          right-ascension in deciaml degrees or sexegesimal format
    dec, dec1, dec2       declination in deciaml degrees or sexegesimal format
    datetime              modified julian date (mjd) or universal time (UT). UT can be formated 20150415113334.343 or "20150415 11:33:34.343" (spaces require quotes)
    north, east           vector components in arcsec
    distVal               a distance value in Mpc (-mpc) or redshift (-z)
    hVal                  hubble constant value. Default=70 km/s/Mpc
    OmegaMatter           Omega Matter. Default=0.3
    OmegaVacuum           Omega Vacuum. Default=0.7

Options:
    init                                    setup the astrocalc settings file for the first time
    -v, --version                           show version
    -h, --help                              show this help message
    -m, --mpc                               distance in mpc
    -z, --redshift                          redshift distance
    -c, --cartesian                         convert to cartesian coordinates
    -s, --settings <pathToSettingsFile>     the settings file
"""
from __future__ import print_function
from builtins import str
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
from docopt import docopt
from fundamentals import tools, times
from astrocalc.coords import unit_conversion

def tab_complete(text, state):
    return (glob.glob(text + '*') + [None])[state]

def main(arguments=None):
    """
    *The main function used when `cl_utils.py` is run as a single script from the cl, or when installed as a cl command*
    """
    from astrocalc.coords import unit_conversion
    # setup the command-line util settings
    su = tools(
        arguments=arguments,
        docString=__doc__,
        logLevel="CRITICAL",
        options_first=True,
        projectName="astrocalc",
        defaultSettingsFile=True
    )
    arguments, settings, log, dbConn = su.setup()

    # tab completion for raw_input
    readline.set_completer_delims(' \t\n;')
    readline.parse_and_bind("tab: complete")
    readline.set_completer(tab_complete)

    # UNPACK REMAINING CL ARGUMENTS USING `EXEC` TO SETUP THE VARIABLE NAMES
    # AUTOMATICALLY
    a = {}
    for arg, val in list(arguments.items()):
        if arg[0] == "-":
            varname = arg.replace("-", "") + "Flag"
        else:
            varname = arg.replace("<", "").replace(">", "")
        a[varname] = val
        if arg == "--dbConn":
            dbConn = val
            a["dbConn"] = val
        log.debug('%s = %s' % (varname, val,))

    ## START LOGGING ##
    startTime = times.get_now_sql_datetime()
    log.info(
        '--- STARTING TO RUN THE cl_utils.py AT %s' %
        (startTime,))

    # set options interactively if user requests
    if "interactiveFlag" in a and a["interactiveFlag"]:

        # load previous settings
        moduleDirectory = os.path.dirname(__file__) + "/resources"
        pathToPickleFile = "%(moduleDirectory)s/previousSettings.p" % locals()
        try:
            with open(pathToPickleFile):
                pass
            previousSettingsExist = True
        except:
            previousSettingsExist = False
        previousSettings = {}
        if previousSettingsExist:
            previousSettings = pickle.load(open(pathToPickleFile, "rb"))

        # x-raw-input
        # x-boolean-raw-input
        # x-raw-input-with-default-value-from-previous-settings

        # save the most recently used requests
        pickleMeObjects = []
        pickleMe = {}
        theseLocals = locals()
        for k in pickleMeObjects:
            pickleMe[k] = theseLocals[k]
        pickle.dump(pickleMe, open(pathToPickleFile, "wb"))

    if a["init"]:
        from os.path import expanduser
        home = expanduser("~")
        filepath = home + "/.config/astrocalc/astrocalc.yaml"
        try:
            cmd = """open %(filepath)s""" % locals()
            p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        except:
            pass
        try:
            cmd = """start %(filepath)s""" % locals()
            p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        except:
            pass
        return

    coordflip = a["coordflip"]
    sep = a["sep"]
    timeflip = a["timeflip"]
    trans = a["trans"]
    now = a["now"]
    dist = a["dist"]
    ra = a["ra"]
    ra1 = a["ra1"]
    ra2 = a["ra2"]
    dec = a["dec"]
    dec1 = a["dec1"]
    dec2 = a["dec2"]
    datetime = a["datetime"]
    north = a["north"], east
    distVal = a["distVal"]
    hVal = a["hVal"]
    OmegaMatter = a["OmegaMatter"]
    OmegaVacuum = a["OmegaVacuum"]
    mpcFlag = a["mpcFlag"]
    redshiftFlag = a["redshiftFlag"]
    cartesianFlag = a["cartesianFlag"]

    # CALL FUNCTIONS/OBJECTS
    if coordflip:

        if cartesianFlag:
            converter = unit_conversion(
                log=log
            )
            x, y, z = converter.ra_dec_to_cartesian(
                ra="23 45 21.23232",
                dec="+01:58:5.45341"
            )
            print(x, y, z)
            return

        try:
            ra = float(ra)
            dec = float(dec)
            degree = True
        except Exception as e:
            degree = False

        if degree is True:
            converter = unit_conversion(
                log=log
            )
            try:
                ra = converter.ra_decimal_to_sexegesimal(
                    ra=ra,
                    delimiter=":"
                )
                dec = converter.dec_decimal_to_sexegesimal(
                    dec=dec,
                    delimiter=":"
                )
            except Exception as e:
                print(e)
                sys.exit(0)

            print(ra, dec)
        else:
            converter = unit_conversion(
                log=log
            )
            try:
                ra = converter.ra_sexegesimal_to_decimal(
                    ra=ra
                )
                dec = converter.dec_sexegesimal_to_decimal(
                    dec=dec
                )
            except Exception as e:
                print(e)
                sys.exit(0)
            print(ra, dec)

    if sep:
        from astrocalc.coords import separations
        calculator = separations(
            log=log,
            ra1=ra1,
            dec1=dec1,
            ra2=ra2,
            dec2=dec2,
        )
        angularSeparation, north, east = calculator.get()
        print("""%(angularSeparation)s arcsec (%(north)s N, %(east)s E)""" % locals())

    if timeflip:
        try:
            inputMjd = float(datetime)
            if datetime[0] not in ["0", "1", "2"]:
                inputMjd = True
            else:
                inputMjd = False
        except:
            inputMjd = False
        from astrocalc.times import conversions
        converter = conversions(
            log=log
        )

        if inputMjd == False:
            try:
                mjd = converter.ut_datetime_to_mjd(utDatetime=datetime)
                print(mjd)
            except Exception as e:
                print(e)
        else:
            try:
                utDate = converter.mjd_to_ut_datetime(mjd=datetime)
                print(utDate)
            except Exception as e:
                print(e)

    if trans:
        # TRANSLATE COORDINATES ACROSS SKY
        from astrocalc.coords import translate
        newRa, newDec = translate(
            log=log,
            ra=ra,
            dec=dec,
            northArcsec=float(north),
            eastArcsec=float(east)
        ).get()
        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        ra = converter.ra_decimal_to_sexegesimal(
            ra=newRa,
            delimiter=":"
        )
        dec = converter.dec_decimal_to_sexegesimal(
            dec=newDec,
            delimiter=":"
        )

        print("%(newRa)s, %(newDec)s (%(ra)s, %(dec)s)" % locals())

    if now:
        from astrocalc.times import now
        mjd = now(
            log=log
        ).get_mjd()
        print(mjd)

    if dist and redshiftFlag:
        from astrocalc.distances import converter
        c = converter(log=log)
        if not hcFlag:
            hcFlag = 70.
        if not wmFlag:
            wmFlag = 0.3
        if not wvFlag:
            wvFlag = 0.7
        dists = c.redshift_to_distance(
            z=float(distVal),
            WM=float(wmFlag),
            WV=float(wvFlag),
            H0=float(hcFlag)
        )
        print("Distance Modulus: " + str(dists["dmod"]) + " mag")
        print("Luminousity Distance: " + str(dists["dl_mpc"]) + " Mpc")
        print("Angular Size Scale: " + str(dists["da_scale"]) + " kpc/arcsec")
        print("Angular Size Distance: " + str(dists["da_mpc"]) + " Mpc")
        print("Comoving Radial Distance: " + str(dists["dcmr_mpc"]) + " Mpc")

    if dist and mpcFlag:
        from astrocalc.distances import converter
        c = converter(log=log)
        z = c.distance_to_redshift(
            mpc=float(distVal)
        )
        print("z = %(z)s" % locals())

    if "dbConn" in locals() and dbConn:
        dbConn.commit()
        dbConn.close()
    ## FINISH LOGGING ##
    endTime = times.get_now_sql_datetime()
    runningTime = times.calculate_time_difference(startTime, endTime)
    log.info('-- FINISHED ATTEMPT TO RUN THE cl_utils.py AT %s (RUNTIME: %s) --' %
             (endTime, runningTime, ))

    return

if __name__ == '__main__':
    main()



#!/usr/local/bin/python
# encoding: utf-8
"""
*Calculations to translate coordinates across the sky*

:Author:
    David Young
"""
from __future__ import division
from builtins import object
from past.utils import old_div
import sys
import os
import math
os.environ['TERM'] = 'vt100'
from fundamentals import tools
from astrocalc.coords import unit_conversion

class translate(object):
    """
    *Translate a set of coordinates north and east by distances given in arcsecs*

    **Key Arguments**

    - ``log`` -- logger
    - ``settings`` -- the settings dictionary. Default *False*
    - ``ra`` -- ra (decimal or sexegesimal)
    - ``dec`` -- dec (decimal or sexegesimal)
    - ``northArcsec`` -- number of arcsecs to move location to the north
    - ``eastArcsec`` -- number of arcsecs to move location to the east
    

    .. todo::

        - replace shift_coordinates class in all other code

    **Usage**

    To shift a set of coordinates north and east by given distances:

    ```python
    # TRANSLATE COORDINATES ACROSS SKY
    from astrocalc.coords import translate
    ra, dec = translate(
        log=log,
        settings=settings,
        ra="14.546438",
        dec="-45.34232334",
        northArcsec=4560,
        eastArcsec=+967800
    ).get()  
    ```
    
    """
    # Initialisation

    def __init__(
            self,
            log,
            ra,
            dec,
            northArcsec,
            eastArcsec,
            settings=False,
    ):
        self.log = log
        log.debug("instansiating a new 'translate' object")
        self.settings = settings
        self.ra = ra
        self.dec = dec
        self.north = northArcsec / 3600.
        self.east = eastArcsec / 3600.
        # xt-self-arg-tmpx

        # CONSTANTS
        self.pi = (4 * math.atan(1.0))
        self.DEG_TO_RAD_FACTOR = self.pi / 180.0
        self.RAD_TO_DEG_FACTOR = 180.0 / self.pi

        # INITIAL ACTIONS
        # CONVERT RA AND DEC INTO DECIMAL DEGREES
        converter = unit_conversion(
            log=log
        )
        self.ra = converter.ra_sexegesimal_to_decimal(
            ra=self.ra
        )
        self.dec = converter.dec_sexegesimal_to_decimal(
            dec=self.dec
        )

        return None

    def get(self):
        """
        *translate the coordinates*

        **Return**

        - ``ra`` -- the right-ascension of the translated coordinate
        - ``dec`` -- the declination of the translated coordinate
        
        """
        self.log.debug('starting the ``get`` method')

        # PRECISION TEST
        decprecision = len(repr(self.dec).split(".")[-1])
        raprecision = len(repr(self.ra).split(".")[-1])

        dec2 = self.dec + self.north

        ra2 = self.ra + \
            (old_div((self.east),
             (math.cos((self.dec + dec2) * self.DEG_TO_RAD_FACTOR / 2.))))

        # FIX VALUES THAT CROSS RA/DEC LIMITS
        while ra2 > 360. or ra2 < 0.:
            while ra2 > 360.:
                ra2 = ra2 - 360.
            while ra2 < 0.:
                ra2 = ra2 + 360.
        while dec2 > 90. or dec2 < -90.:
            while dec2 > 90.:
                dec2 = 180. - dec2
            while dec2 < -90.:
                dec2 = -180. - dec2

        ra2 = "%0.*f" % (raprecision, ra2)
        dec2 = "%0.*f" % (decprecision, dec2)

        self.log.debug('completed the ``get`` method')
        return ra2, dec2

    # xt-class-method



#!/usr/local/bin/python
# encoding: utf-8
"""
*Convert single values of RA, DEC or list of RA and DEC to numpy arrays*

:Author:
    David Young
"""
import sys
import os
os.environ['TERM'] = 'vt100'
from fundamentals import tools
import numpy as np
from astrocalc.coords import unit_conversion

def coordinates_to_array(
        log,
        ra,
        dec):
    """*Convert a single value RA, DEC or list of RA and DEC to numpy arrays*

    **Key Arguments**

    - ``ra`` -- list, numpy array or single ra value
    - ``dec`` --list, numpy array or single dec value
    - ``log`` -- logger
    

    **Return**

    - ``raArray`` -- input RAs as a numpy array of decimal degree values
    - ``decArray`` -- input DECs as a numpy array of decimal degree values
    

    **Usage**

    .. todo::

        add usage info
        create a sublime snippet for usage

    ```python
    ra, dec = coordinates_to_array(
        log=log,
        ra=ra,
        dec=dec
    )
    ```
    
    """
    log.debug('starting the ``coordinates_to_array`` function')

    if isinstance(ra, np.ndarray) and isinstance(dec, np.ndarray):
        return ra, dec

    # ASTROCALC UNIT CONVERTER OBJECT
    converter = unit_conversion(
        log=log
    )
    # CONVERT RA AND DEC TO NUMPY ARRAYS
    if isinstance(ra, float):
        pass
    elif isinstance(ra, ("".__class__, u"".__class__)):
        try:
            ra = float(ra)
        except:
            ra = converter.ra_sexegesimal_to_decimal(ra=ra)
    elif isinstance(ra, list):
        try:
            ra = np.array(ra).astype(np.float)
        except:
            raList = []
            raList[:] = [converter.ra_sexegesimal_to_decimal(ra=r) for r in ra]
            ra = raList

    if isinstance(dec, float):
        pass
    elif isinstance(dec, ("".__class__, u"".__class__)):
        try:
            dec = float(dec)
        except:
            dec = converter.dec_sexegesimal_to_decimal(dec=dec)
    elif isinstance(dec, list):
        try:
            dec = np.array(dec).astype(np.float)
        except:
            decList = []
            decList[:] = [
                converter.dec_sexegesimal_to_decimal(dec=d) for d in dec]
            dec = decList

    raArray = np.array(ra, dtype='f8', ndmin=1, copy=False)
    decArray = np.array(dec, dtype='f8', ndmin=1, copy=False)

    log.debug('completed the ``coordinates_to_array`` function')
    return raArray, decArray



#!/usr/local/bin/python
# encoding: utf-8
"""
*Report current time in various formats*

:Author:
    David Young
"""
from __future__ import division
from past.utils import old_div
from builtins import object
import sys
import os
import math
import time
os.environ['TERM'] = 'vt100'
from fundamentals import tools

class now(object):
    """
    *Report the current time into various formats*

    **Key Arguments**

    - ``log`` -- logger
    - ``settings`` -- the settings dictionary
    
    """
    # Initialisation

    def __init__(
            self,
            log,
            settings=False,

    ):
        self.log = log
        log.debug("instansiating a new 'now' object")
        self.settings = settings
        # xt-self-arg-tmpx

        # Initial Actions

        return None

    def get_mjd(self):
        """
        *Get the current time as an MJD*

        **Return**

        - ``mjd`` -- the current MJD as a float
        

        **Usage**

        
        .. todo::

            - add clutil
            - remove `getCurrentMJD` from all other code

        ```python
        from astrocalc.times import now
        mjd = now(
            log=log
        ).get_mjd()
        ```
        """
        self.log.debug('starting the ``get_mjd`` method')

        jd = old_div(time.time(), 86400.0) + 2440587.5
        mjd = jd - 2400000.5

        self.log.debug('completed the ``get_mjd`` method')
        return mjd

    # xt-class-method



from __future__ import print_function
from builtins import str
import os
import unittest
import shutil
import unittest
import yaml
from astrocalc.utKit import utKit
from fundamentals import tools
from os.path import expanduser
home = expanduser("~")

packageDirectory = utKit("").get_project_root()
settingsFile = packageDirectory + "/test_settings.yaml"
su = tools(
    arguments={"settingsFile": settingsFile},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName=None,
    defaultSettingsFile=False
)
arguments, settings, log, dbConn = su.setup()

# SETUP PATHS TO COMMON DIRECTORIES FOR TEST DATA
moduleDirectory = os.path.dirname(__file__)
pathToInputDir = moduleDirectory + "/input/"
pathToOutputDir = moduleDirectory + "/output/"

try:
    shutil.rmtree(pathToOutputDir)
except:
    pass
# COPY INPUT TO OUTPUT DIR
shutil.copytree(pathToInputDir, pathToOutputDir)

# Recursively create missing directories
if not os.path.exists(pathToOutputDir):
    os.makedirs(pathToOutputDir)


class test_conversions(unittest.TestCase):

    def test_utdatetime_conversions_function(self):

        from astrocalc.times import conversions
        converter = conversions(
            log=log,
        )

        print("\nUT = 20160426t144643.033433")
        print(converter.ut_datetime_to_mjd(
            utDatetime="20160426t144643.033433"))
        print(converter.mjd_to_ut_datetime(mjd=57504.61577585013))

        print("\nUT = 20160426t144643.033433")
        print(converter.ut_datetime_to_mjd(
            utDatetime="20160426t144643.033433"))
        print(converter.mjd_to_ut_datetime(mjd=57504.61577585013))

        print("\nUT = 20160426t144643.033433")
        print(converter.ut_datetime_to_mjd(
            utDatetime="20160426t144643.033433"))
        print(converter.mjd_to_ut_datetime(mjd=57504.61577585013))

        print("\nUT = 20160426t1446")
        print(converter.ut_datetime_to_mjd(utDatetime="20160426t1446"))
        print(converter.mjd_to_ut_datetime(mjd=57504.6153))
        print("\nUT = 20160426t144643")
        print(converter.ut_datetime_to_mjd(utDatetime="20160426t144643"))
        print(converter.mjd_to_ut_datetime(mjd=57504.61578))
        print("\nUT = 20160426t144643.033433")
        print(converter.ut_datetime_to_mjd(
            utDatetime="20160426t144643.033433"))
        print(converter.mjd_to_ut_datetime(mjd=57504.61577585013))
        print("\nUT = 20161231t234643.033433")
        print(converter.ut_datetime_to_mjd(
            utDatetime="20161231t234643.033433"))
        print(converter.ut_datetime_to_mjd(
            utDatetime="20161231t234643.033433"))
        print(converter.mjd_to_ut_datetime(mjd="57753.99077585013"))
        print("\nUT = 201604261444")
        print(converter.ut_datetime_to_mjd(utDatetime="201604261444"))
        print(converter.mjd_to_ut_datetime(mjd=57504.6139))
        print("\nUT = 20160426")
        print(converter.ut_datetime_to_mjd(utDatetime="20160426"))
        print(converter.mjd_to_ut_datetime(mjd=57504.0))
        print("\nUT = 2016-04-26.33433")
        print(converter.ut_datetime_to_mjd(utDatetime="2016-04-26.33433"))
        print(converter.mjd_to_ut_datetime(mjd=57504.33411))
        print("\nUT = 20160426144444.5452")
        print(converter.ut_datetime_to_mjd(utDatetime="20160426144444.5452"))
        print(converter.mjd_to_ut_datetime(mjd=57504.614404459))
        print("\nUT = 2016-04-26 14:44:44.234")
        print(converter.ut_datetime_to_mjd(
            utDatetime="2016-04-26 14:44:44.234"))
        print(converter.mjd_to_ut_datetime(mjd=57504.61440086))
        print("\nUT = 20160426 14h44m44.432s")
        print(converter.ut_datetime_to_mjd(
            utDatetime="20160426 14h44m44.432s"))
        print(converter.mjd_to_ut_datetime(mjd=57504.61440315))
        print("\nUT = 2016-04-26T14:44:44.234")
        print(converter.ut_datetime_to_mjd(
            utDatetime="2016-04-26T14:44:44.234"))
        print(converter.mjd_to_ut_datetime(
            mjd=57504.61440086,
            sqlDate=True
        ))

    def test_mjd_conversions_function(self):

        from astrocalc.times import conversions
        converter = conversions(
            log=log,
        )
        print(converter.mjd_to_ut_datetime(mjd=57504.61440))
        print(converter.mjd_to_ut_datetime(mjd=57504.61440))
        print(converter.mjd_to_ut_datetime(mjd=57504.61440))
        print(converter.mjd_to_ut_datetime(mjd=57504.61440))
        print(converter.mjd_to_ut_datetime(mjd=57504.61440))
        print(converter.mjd_to_ut_datetime(mjd=57504.61440))
        print(converter.mjd_to_ut_datetime(mjd=57504.61440))
        print(converter.mjd_to_ut_datetime(mjd=57504.61440))
        print(converter.mjd_to_ut_datetime(mjd=57504.61440))
        print(converter.mjd_to_ut_datetime(mjd=57504.61440))

    def test_decimal_day_to_day_hour_min_sec_function(self):

        from astrocalc.times import conversions
        converter = conversions(
            log=log,
        )
        daysInt, hoursInt, minsInt, secFloat = converter.decimal_day_to_day_hour_min_sec(
            daysFloat=24.2453)
        print("%(daysInt)s days, %(hoursInt)s hours, %(minsInt)s mins, %(secFloat)s sec" % locals())

        daysInt, hoursInt, minsInt, secFloat = converter.decimal_day_to_day_hour_min_sec(
            daysFloat=24.12345)
        print("%(daysInt)s days, %(hoursInt)s hours, %(minsInt)s mins, %(secFloat)s sec" % locals())

        daysInt, hoursInt, minsInt, secFloat = converter.decimal_day_to_day_hour_min_sec(
            daysFloat=24.2)
        print("%(daysInt)s days, %(hoursInt)s hours, %(minsInt)s mins, %(secFloat)s sec" % locals())

        daysInt, hoursInt, minsInt, secFloat = converter.decimal_day_to_day_hour_min_sec(
            daysFloat=24.1232435454)
        print("%(daysInt)s days, %(hoursInt)s hours, %(minsInt)s mins, %(secFloat)s sec" % locals())

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function



#!/usr/local/bin/python
# encoding: utf-8
"""
*Convert distances between measurement scales*

:Author:
    David Young
"""
from __future__ import division
from builtins import range
from builtins import object
from past.utils import old_div
import sys
import os
import math
os.environ['TERM'] = 'vt100'
from fundamentals import tools

class converter(object):
    """
    *A converter to switch distance between various units of measurement*

    **Key Arguments**

    - ``log`` -- logger
    - ``settings`` -- the settings dictionary
    

    **Usage**

    To instantiate a ``converter`` object:

    ```python
    from astrocalc.distances import converter
    c = converter(log=log)
    ```
    
    """
    # Initialisation

    def __init__(
            self,
            log,
            settings=False,

    ):
        self.log = log
        log.debug("instansiating a new 'converter' object")
        self.settings = settings
        # xt-self-arg-tmpx

        # Initial Actions

        return None

    def distance_to_redshift(
            self,
            mpc):
        """*Convert a distance from MPC to redshift*

        The code works by iteratively converting a redshift to a distance, correcting itself and honing in on the true answer (within a certain precision)

        **Key Arguments**

        - ``mpc`` -- distance in MPC (assumes a luminousity distance).
        

        **Return**

        - ``redshift``
        

        .. todo::

            - replace convert_mpc_to_redshift in all code

        **Usage**

        ```python
        from astrocalc.distances import converter
        c = converter(log=log)
        z = c.distance_to_redshift(
            mpc=500
        )

        print(z)

        # OUTPUT: 0.108
        ```
        
        """
        self.log.debug('starting the ``distance_to_redshift`` method')

        lowerLimit = 0.
        upperLimit = 30.
        redshift = upperLimit - lowerLimit
        distGuess = float(self.redshift_to_distance(redshift)['dl_mpc'])

        distDiff = mpc - distGuess

        while math.fabs(distDiff) > 0.0001:
            if distGuess < mpc:
                lowerLimit = redshift
                redshift = lowerLimit + (upperLimit - lowerLimit) / 2.
                distGuess = float(
                    self.redshift_to_distance(redshift)['dl_mpc'])
            elif distGuess > mpc:
                upperLimit = redshift
                redshift = lowerLimit + (upperLimit - lowerLimit) / 2.
                distGuess = float(
                    self.redshift_to_distance(redshift)['dl_mpc'])
            distDiff = mpc - distGuess

        redshift = float("%5.4f" % (redshift,))

        self.log.debug('completed the ``distance_to_redshift`` method')
        return redshift

    def redshift_to_distance(
            self,
            z,
            WM=0.3,
            WV=0.7,
            H0=70.0):
        """*convert redshift to various distance measurements*

        **Key Arguments**

        - ``z`` -- redshift measurement.
        - ``WM`` -- Omega_matter. Default *0.3*
        - ``WV`` -- Omega_vacuum. Default *0.7*
        - ``H0`` -- Hubble constant. (km s-1 Mpc-1) Default *70.0*
        

        **Return**

        - ``results`` -- result dictionary including
            - ``dcmr_mpc`` -- co-moving radius distance
            - ``da_mpc`` -- angular distance
            - ``da_scale`` -- angular distance scale
            - ``dl_mpc`` -- luminosity distance (usually use this one)
            - ``dmod`` -- distance modulus (determined from luminosity distance)
        

        ..  todo::

                - replace convert_redshift_to_distance in all other code

        **Usage**

        ```python
        from astrocalc.distances import converter
        c = converter(log=log)
        dists = c.redshift_to_distance(
            z=0.343
        )

        print("Distance Modulus: " + str(dists["dmod"]) + " mag")
        print("Luminousity Distance: " + str(dists["dl_mpc"]) + " Mpc")
        print("Angular Size Scale: " + str(dists["da_scale"]) + " kpc/arcsec")
        print("Angular Size Distance: " + str(dists["da_mpc"]) + " Mpc")
        print("Comoving Radial Distance: " + str(dists["dcmr_mpc"]) + " Mpc")

        # OUTPUT :
        # Distance Modulus: 41.27 mag
        # Luminousity Distance: 1795.16 Mpc
        # Angular Size Scale: 4.85 kpc/arcsec
        # Angular Size Distance: 999.76 Mpc
        # Comoving Radial Distance: 1339.68 Mpc

        from astrocalc.distances import converter
        c = converter(log=log)
        dists = c.redshift_to_distance(
            z=0.343,
            WM=0.286,
            WV=0.714,
            H0=69.6
        )

        print("Distance Modulus: " + str(dists["dmod"]) + " mag")
        print("Luminousity Distance: " + str(dists["dl_mpc"]) + " Mpc")
        print("Angular Size Scale: " + str(dists["da_scale"]) + " kpc/arcsec")
        print("Angular Size Distance: " + str(dists["da_mpc"]) + " Mpc")
        print("Comoving Radial Distance: " + str(dists["dcmr_mpc"]) + " Mpc")

        # OUTPUT :
        # Distance Modulus: 41.29 mag
        # Luminousity Distance: 1811.71 Mpc
        # Angular Size Scale: 4.89 kpc/arcsec
        # Angular Size Distance: 1008.97 Mpc
        # Comoving Radial Distance: 1352.03 Mpc
        ```
        
        """
        self.log.debug('starting the ``redshift_to_distance`` method')

        # VARIABLE
        h = H0 / 100.0
        WR = old_div(4.165E-5, (h * h))     # Omega_radiation
        WK = 1.0 - WM - WV - WR       # Omega_curvature = 1 - Omega(Total)
        c = 299792.458          # speed of light (km/s)

        # Arbitrarily set the values of these variables to zero just so we can
        # define them.
        DCMR = 0.0             # comoving radial distance in units of c/H0
        DCMR_Mpc = 0.0          # comoving radial distance in units of Mpc
        DA = 0.0                # angular size distance in units of c/H0
        DA_Mpc = 0.0            # angular size distance in units of Mpc
        # scale at angular size distance in units of Kpc / arcsec
        DA_scale = 0.0
        DL = 0.0                # luminosity distance in units of c/H0
        DL_Mpc = 0.0            # luminosity distance in units of Mpc
        # Distance modulus determined from luminosity distance
        DMOD = 0.0
        a = 0.0                 # 1/(1+z), the scale factor of the Universe

        az = 1.0 / (1.0 + z)        # 1/(1+z), for the given redshift

        # Compute the integral over a=1/(1+z) from az to 1 in n steps
        n = 1000
        for i in range(n):
            a = az + old_div((1.0 - az) * (i + 0.5), n)
            adot = math.sqrt(WK + (old_div(WM, a)) + (old_div(WR, (math.pow(a, 2))))
                             + (WV * math.pow(a, 2)))
            DCMR = DCMR + 1.0 / (a * adot)

        # comoving radial distance in units of c/H0
        DCMR = old_div((1.0 - az) * DCMR, n)
        # comoving radial distance in units of Mpc
        DCMR_Mpc = (old_div(c, H0)) * DCMR

        # Tangental comoving radial distance
        x = math.sqrt(abs(WK)) * DCMR
        if x > 0.1:
            if WK > 0.0:
                ratio = old_div(0.5 * (math.exp(x) - math.exp(-x)), x)
            else:
                ratio = old_div(math.sin(x), x)
        else:
            y = math.pow(x, 2)
            if WK < 0.0:
                y = -y
            ratio = 1 + y / 6.0 + math.pow(y, 2) / 120.0

        DA = az * ratio * DCMR  # angular size distance in units of c/H0
        DA_Mpc = (old_div(c, H0)) * DA  # angular size distance in units of Mpc
        # scale at angular size distance in units of Kpc / arcsec
        DA_scale = DA_Mpc / 206.264806
        DL = old_div(DA, math.pow(az, 2))  # luminosity distance in units of c/H0
        DL_Mpc = (old_div(c, H0)) * DL  # luminosity distance in units of Mpc
        # Distance modulus determined from luminosity distance
        DMOD = 5 * math.log10(DL_Mpc * 1e6) - 5

        # FIXING PRECISIONS
        # PRECISION TEST
        precision = len(repr(z).split(".")[-1])
        DCMR_Mpc = "%0.*f" % (precision, DCMR_Mpc)
        DA_Mpc = "%0.*f" % (precision, DA_Mpc)
        DA_scale = "%0.*f" % (precision, DA_scale)
        DL_Mpc = "%0.*f" % (precision, DL_Mpc)
        DMOD = "%0.*f" % (precision, DMOD)
        z = "%0.*f" % (precision, z)

        results = \
            {
                "dcmr_mpc": float(DCMR_Mpc),
                "da_mpc": float(DA_Mpc),
                "da_scale": float(DA_scale),
                "dl_mpc": float(DL_Mpc),
                "dmod": float(DMOD),
                "z": float(z)
            }

        self.log.debug('completed the ``redshift_to_distance`` method')
        return results

    # use the tab-trigger below for new method
    # xt-class-method



#!/usr/local/bin/python
# encoding: utf-8
"""
*Calculate separations between sky-coordinates*

:Author:
    David Young
"""
from __future__ import division
from past.utils import old_div
from builtins import object
import sys
import os
import math
os.environ['TERM'] = 'vt100'
from fundamentals import tools

class separations(object):
    """
    *The worker class for the separations module*

    **Key Arguments**

    - ``log`` -- logger
    - ``settings`` -- the settings dictionary
    - ``ra1`` -- the right-ascension of the first location. Decimal degrees or sexegesimal.
    - ``dec1`` -- the declination of the first location. Decimal degrees or sexegesimal.
    - ``ra2`` -- the right-ascension of the second location. Decimal degrees or sexegesimal.
    - ``dec2`` -- the declination of the second location. Decimal degrees or sexegesimal.
    

    **Usage**

    .. todo::

        - replace get_angular_separation throughout all code using dryxPython
        - replace getAngularSeparationthroughout all code using dryxPython

    You can input sexegesimal coordinates,

    ```python
    from astrocalc.coords import separations
    calculator = separations(
        log=log,
        ra1="23:32:23.2324",
        dec1="-13:32:45.43553",
        ra2="23:32:34.642",
        dec2="-12:12:34.9334",
    )
    angularSeparation, north, east = calculator.get()
    print(angularSeparation, north, east)

    # OUT: '4813.39431', '4810.50214', '166.83941'
    ```

    or decimal degrees,

    ```python
    from astrocalc.coords import separations
    calculator = separations(
        log=log,
        ra1=2.3342343,
        dec1=89.23244233,
        ra2=45.343545345,
        dec2=87.3435435
    )
    angularSeparation, north, east = calculator.get()
    print(angularSeparation, north, east)

    # OUT: '7774.4375', '-6800.0358', '4625.7620'
    ```

    or even a mix of both

    ```python
    from astrocalc.coords import separations
    calculator = separations(
        log=log,
        ra1=352.5342343,
        dec1=89.23,
        ra2="23:32:34.642",
        dec2="89:12:34.9334"
    )
    angularSeparation, north, east = calculator.get()
    print(angularSeparation, north, east)

    # OUT: '78.9', '-73.1', '29.9')
    ```
    
    """
    # Initialisation

    def __init__(
            self,
            log,
            ra1,
            dec1,
            ra2,
            dec2,
            settings=False
    ):
        self.log = log
        log.debug("instansiating a new 'separations' object")
        self.settings = settings
        self.ra1 = ra1
        self.dec1 = dec1
        self.ra2 = ra2
        self.dec2 = dec2
        # xt-self-arg-tmpx
        return None

    def get(
            self):
        """*Calulate the angular separation between two locations on the sky*

        Input precision should be respected.

        **Key Arguments**

        

        **Return**

        - ``angularSeparation`` -- total angular separation between coordinates (arcsec)
        - ``north`` -- north-south separation between coordinates (arcsec)
        - ``east`` -- east-west separation between coordinates (arcsec)
        

        See main class usage for details.
        """
        self.log.debug('starting the ``get_angular_separation`` method')

        from astrocalc.coords import unit_conversion

        # CONSTANTS
        pi = (4 * math.atan(1.0))
        DEG_TO_RAD_FACTOR = old_div(pi, 180.0)
        RAD_TO_DEG_FACTOR = old_div(180.0, pi)

        converter = unit_conversion(
            log=self.log
        )
        dec1 = converter.dec_sexegesimal_to_decimal(
            dec=self.dec1
        )
        dec2 = converter.dec_sexegesimal_to_decimal(
            dec=self.dec2
        )
        ra1 = converter.ra_sexegesimal_to_decimal(
            ra=self.ra1
        )
        ra2 = converter.ra_sexegesimal_to_decimal(
            ra=self.ra2
        )

        # PRECISION TEST
        precision = 100
        vals = [dec1, dec2, ra1, ra2]
        for v in vals:
            thisLen = len(repr(v * 3600.).split(".")[-1])
            if thisLen < precision:
                precision = thisLen

        angularSeparation = None

        aa = (90.0 - dec1) * DEG_TO_RAD_FACTOR
        bb = (90.0 - dec2) * DEG_TO_RAD_FACTOR
        cc = (ra1 - ra2) * DEG_TO_RAD_FACTOR
        one = math.cos(aa) * math.cos(bb)
        two = math.sin(aa) * math.sin(bb) * math.cos(cc)

        # Because acos() returns NaN outside the ranges of -1 to +1
        # we need to check this.  Double precision decimal places
        # can give values like 1.0000000000002 which will throw an
        # exception.

        three = one + two
        if (three > 1.0):
            three = 1.0
        if (three < -1.0):
            three = -1.0

        # BE CAREFUL WITH PRECISION PROPAGATION
        thisVal = math.acos(three)
        angularSeparation = float(thisVal) * RAD_TO_DEG_FACTOR * 3600.0

        # Now work out N-S, E-W separations (object 1 relative to 2)
        north = -(dec1 - dec2) * 3600.0
        east = -(ra1 - ra2) * \
            math.cos((dec1 + dec2) * DEG_TO_RAD_FACTOR / 2.) * 3600.0

        angularSeparation = "%0.*f" % (precision, angularSeparation)
        north = "%0.*f" % (precision, north)
        east = "%0.*f" % (precision, east)

        self.log.debug('completed the ``get_angular_separation`` method')
        return angularSeparation, north, east

    # use the tab-trigger below for new method
    # xt-class-method



#!/usr/local/bin/python
# encoding: utf-8
"""
*Convert times between various epochs and units*

:Author:
    David Young
"""
from __future__ import division
from past.utils import old_div
from builtins import object
import sys
import os
os.environ['TERM'] = 'vt100'
from fundamentals import tools

class conversions(object):
    """
    *The worker class for the conversions module*

    **Key Arguments**

    - ``log`` -- logger
    - ``settings`` -- the settings dictionary
    

    **Usage**

    .. todo::

        - add usage info
        - create a sublime snippet for usage
        - add mjd_to_date
        - add decimal_day_to_day_hour_min_sec
        - add date_to_mjd
        - convert all functions in __init__ to modules

    ```python
    usage code   
    ```
    
    """
    # Initialisation

    def __init__(
            self,
            log,
            settings=False,
    ):
        self.log = log
        log.debug("instansiating a new 'conversions' object")
        self.settings = settings
        # xt-self-arg-tmpx

        return None

    def get(self):
        """
        *get the conversions object*

        **Return**

        - ``conversions``
        

        .. todo::

            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.debug('starting the ``get`` method')

        conversions = None

        self.log.debug('completed the ``get`` method')
        return conversions

    def ut_datetime_to_mjd(
            self,
            utDatetime):
        """*ut datetime to mjd*

        If the date given has no time associated with it (e.g. ``20160426``), then the datetime assumed is ``20160426 00:00:00.0``.

        Precision should be respected. 

        **Key Arguments**

        - ``utDatetime`` -- UT datetime. Can accept various formats e.g. ``201604261444``, ``20160426``, ``20160426144444.5452``, ``2016-04-26 14:44:44.234``, ``20160426 14h44m44.432s`` 
        

        **Return**

        - ``mjd`` -- the MJD
        

        .. todo ::

            - replace getMJDFromSqlDate in all code

        **Usage**

        ```python
        from astrocalc.times import conversions
        converter = conversions(
            log=log
        )
        mjd = converter.ut_datetime_to_mjd(utDatetime="20160426t1446")
        print(mjd)

        # OUT: 57504.6153

        mjd = converter.ut_datetime_to_mjd(utDatetime="2016-04-26 14:44:44.234")
        print(mjd)

        # OUT: 57504.61440
        ```
        
        """
        self.log.debug('starting the ``ut_datetime_to_mjd`` method')

        import time
        import re
        mjd = None

        # TRIM WHITESPACE FROM AROUND STRING
        utDatetime = utDatetime.strip()

        # DATETIME REGEX
        matchObject = re.match(
            r'^(?P<year>\d{4})\D?(?P<month>(0\d|1[0-2]))\D?(?P<day>([0-2]\d|3[0-1])(\.\d+)?)(\D?(?P<hours>([0-1]\d|2[0-3]))\D?(?P<minutes>\d{2})(\D?(?P<seconds>\d{2}(\.\d*?)?))?)?s?$', utDatetime)

        # RETURN ERROR IF REGEX NOT MATCHED
        if not matchObject:
            self.log.error(
                'UT Datetime is not in a recognised format. Input value was `%(utDatetime)s`' % locals())
            raise IOError(
                'UT Datetime is not in a recognised format. Input value was `%(utDatetime)s`' % locals())

        year = matchObject.group("year")
        month = matchObject.group("month")
        day = matchObject.group("day")
        hours = matchObject.group("hours")
        minutes = matchObject.group("minutes")
        seconds = matchObject.group("seconds")

        # CLEAN NUMBERS AND SET OUTPUT PRECISION
        if "." in day:
            fhours = (float(day) - int(float(day))) * 24
            hours = int(fhours)
            fminutes = (fhours - hours) * 60
            minutes = int(fminutes)
            seconds = fhours - minutes
            precision = len(repr(day).split(".")[-1])
        elif not hours:
            hours = "00"
            minutes = "00"
            seconds = "00"
            precision = 1
        elif not seconds:
            seconds = "00"
            # PRECISION TO NEAREST MIN i.e. 0.000694444 DAYS
            precision = 4
        else:
            if "." not in seconds:
                precision = 5
            else:
                decLen = len(seconds.split(".")[-1])
                precision = 5 + decLen

        # CONVERT VALUES TO FLOAT
        seconds = float(seconds)
        year = float(year)
        month = float(month)
        day = float(day)
        hours = float(hours)
        minutes = float(minutes)

        # DETERMINE EXTRA TIME (SMALLER THAN A SEC)
        extraTime = 0.
        if "." in repr(seconds):
            extraTime = old_div(float("." + repr(seconds).split(".")
                              [-1]), (24. * 60. * 60.))

        # CONVERT TO UNIXTIME THEN MJD
        t = (int(year), int(month), int(day), int(hours),
             int(minutes), int(seconds), 0, 0, 0)
        unixtime = int(time.mktime(t))
        mjd = (old_div(unixtime, 86400.0) - 2400000.5 + 2440587.5) + extraTime

        mjd = "%0.*f" % (precision, mjd)

        self.log.debug('completed the ``ut_datetime_to_mjd`` method')
        return mjd

    def mjd_to_ut_datetime(
            self,
            mjd,
            sqlDate=False,
            datetimeObject=False):
        """*mjd to ut datetime*

        Precision should be respected. 

        **Key Arguments**

        - ``mjd`` -- time in MJD.
        - ``sqlDate`` -- add a 'T' between date and time instead of space
        - ``datetimeObject`` -- return a datetime object instead of a string. Default *False*
        

        .. todo::

            - replace getDateFromMJD in all code
            - replace getSQLDateFromMJD in all code

        **Return**

        - ``utDatetime`` - the UT datetime in string format
        

        **Usage**

        ```python
        from astrocalc.times import conversions
        converter = conversions(
            log=log
        )
        utDate = converter.mjd_to_ut_datetime(
            mjd=57504.61577585013
        )
        print(utDate)

        # OUT: 2016-04-26 14:46:43.033

        utDate = converter.mjd_to_ut_datetime(
            mjd=57504.61577585013,
            sqlDate=True
        )
        print(utDate)

        # OUT: 2016-04-26T14:46:43.033
        ```
        
        """
        self.log.debug('starting the ``mjd_to_ut_datetime`` method')

        from datetime import datetime

        # CONVERT TO UNIXTIME
        unixtime = (float(mjd) + 2400000.5 - 2440587.5) * 86400.0
        theDate = datetime.utcfromtimestamp(unixtime)

        if datetimeObject == False:
            # DETERMINE PRECISION
            strmjd = repr(mjd)
            if "." not in strmjd:
                precisionUnit = "day"
                precision = 0
                utDatetime = theDate.strftime("%Y-%m-%d")
            else:
                lenDec = len(strmjd.split(".")[-1])
                if lenDec < 2:
                    precisionUnit = "day"
                    precision = 0
                    utDatetime = theDate.strftime("%Y-%m-%d")
                elif lenDec < 3:
                    precisionUnit = "hour"
                    precision = 0
                    utDatetime = theDate.strftime("%Y-%m-%d")
                elif lenDec < 5:
                    precisionUnit = "minute"
                    precision = 0
                    utDatetime = theDate.strftime("%Y-%m-%d %H:%M")
                else:
                    precisionUnit = "second"
                    precision = lenDec - 5
                    if precision > 3:
                        precision = 3
                    secs = float(theDate.strftime("%S.%f"))
                    secs = "%02.*f" % (precision, secs)
                    utDatetime = theDate.strftime("%Y-%m-%d %H:%M:") + secs

            if sqlDate:
                utDatetime = utDatetime.replace(" ", "T")
        else:
            utDatetime = theDate

        self.log.debug('completed the ``mjd_to_ut_datetime`` method')
        return utDatetime

    def decimal_day_to_day_hour_min_sec(
            self,
            daysFloat):
        """*Convert a day from decimal format to hours mins and sec*

        Precision should be respected. 

        **Key Arguments**

        - ``daysFloat`` -- the day as a decimal.
        

        **Return**

        - ``daysInt`` -- day as an integer
        - ``hoursInt`` -- hour as an integer (None if input precsion too low)
        - ``minsInt`` -- mins as an integer (None if input precsion too low)
        - ``secFloat`` -- secs as a float (None if input precsion too low)
        

        **Usage**

        ..  todo::

            - replace `decimal_day_to_day_hour_min_sec` in all other code

        ```python
        from astrocalc.times import conversions
        converter = conversions(
            log=log
        )
        daysInt, hoursInt, minsInt, secFloat = converter.decimal_day_to_day_hour_min_sec(
            daysFloat=24.2453
        )
        print(daysInt, hoursInt, minsInt, secFloat)

        # OUTPUT: 24, 5, 53, None

        daysInt, hoursInt, minsInt, secFloat = converter.decimal_day_to_day_hour_min_sec(
            daysFloat=24.1232435454
        )
        print("%(daysInt)s days, %(hoursInt)s hours, %(minsInt)s mins, %(secFloat)s sec" % locals())

        # OUTPUT: 24 days, 2 hours, 57 mins, 28.242 sec
        ```
        
        """
        self.log.debug(
            'completed the ````decimal_day_to_day_hour_min_sec`` method')

        daysInt = int(daysFloat)
        hoursFloat = (daysFloat - daysInt) * 24.
        hoursInt = int(hoursFloat)
        minsFloat = (hoursFloat - hoursInt) * 60.
        minsInt = int(minsFloat)
        secFloat = (minsFloat - minsInt) * 60.

        # DETERMINE PRECISION
        strday = repr(daysFloat)
        if "." not in strday:
            precisionUnit = "day"
            precision = 0
            hoursInt = None
            minsInt = None
            secFloat = None
        else:
            lenDec = len(strday.split(".")[-1])
            if lenDec < 2:
                precisionUnit = "day"
                precision = 0
                hoursInt = None
                minsInt = None
                secFloat = None
            elif lenDec < 3:
                precisionUnit = "hour"
                precision = 0
                minsInt = None
                secFloat = None
            elif lenDec < 5:
                precisionUnit = "minute"
                precision = 0
                secFloat = None
            else:
                precisionUnit = "second"
                precision = lenDec - 5
                if precision > 3:
                    precision = 3
                secFloat = "%02.*f" % (precision, secFloat)

        self.log.debug(
            'completed the ``decimal_day_to_day_hour_min_sec`` method')
        return daysInt, hoursInt, minsInt, secFloat

    # use the tab-trigger below for new method
    # xt-class-method



from __future__ import print_function
from builtins import str
import os
import unittest
import shutil
import unittest
import yaml
from astrocalc.utKit import utKit
from fundamentals import tools
from os.path import expanduser
home = expanduser("~")

packageDirectory = utKit("").get_project_root()
settingsFile = packageDirectory + "/test_settings.yaml"
su = tools(
    arguments={"settingsFile": settingsFile},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName=None,
    defaultSettingsFile=False
)
arguments, settings, log, dbConn = su.setup()

# SETUP PATHS TO COMMON DIRECTORIES FOR TEST DATA
moduleDirectory = os.path.dirname(__file__)
pathToInputDir = moduleDirectory + "/input/"
pathToOutputDir = moduleDirectory + "/output/"

try:
    shutil.rmtree(pathToOutputDir)
except:
    pass
# COPY INPUT TO OUTPUT DIR
shutil.copytree(pathToInputDir, pathToOutputDir)

# Recursively create missing directories
if not os.path.exists(pathToOutputDir):
    os.makedirs(pathToOutputDir)


class test_coordinates_to_array(unittest.TestCase):

    def test_coordinates_to_array(self):

        raList = ["13:20:00.00", 200.0, "13:20:00.00", 175.23, 21.36]
        decList = ["+24:18:00.00",  24.3,  "+24:18:00.00",  -28.25, -15.32]

        from astrocalc.coords import coordinates_to_array
        ra, dec = coordinates_to_array(
            log=log,
            ra=raList,
            dec=decList
        )

        print(ra, dec)

        from astrocalc.coords import coordinates_to_array
        ra, dec = coordinates_to_array(
            log=log,
            ra="13:20:00.00",
            dec="+24:18:00.00"
        )

        print(ra, dec)

    def test_coordinates_to_array_function_exception(self):

        from astrocalc.coords import coordinates_to_array
        try:
            this = coordinates_to_array(
                log=log,
                settings=settings,
                fakeKey="break the code"
            )
            this.get()
            assert False
        except Exception as e:
            assert True
            print(str(e))

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function



"""
*Testing the rendering of docstings - feel free to remove this module*

Check the API output here: file:///Users/Dave/git_repos/_templates_/python-package-template/astrocalc/docs/build/html/_api/astrocalc.docstring_test.html
"""

def docsting_test(self):
    """
    *the database object for sherlock, setting up ssh tunnels and various database connections*

    The returned dictionary of database connections contain the following databases:

        - ``transients`` -- the database hosting the transient source data
        - ``catalogues`` -- connection to the database hosting the contextual catalogues the transients are to be crossmatched against
        - ``marshall`` -- connection to the PESSTO Marshall database

    **Key Arguments**

    - ``log`` -- logger
    - ``settings`` -- the settings dictionary
    

    **Return**

    - ``dbConns`` -- a dictionary of the database connections required by sherlock
    

    **Usage**

    To setup the sherlock database connections, run the following:

    ```python
    # SETUP ALL DATABASE CONNECTIONS
    from sherlock import database
    db = database(
        log=log,
        settings=settings
    )
    dbConns, dbVersions = db.connect()
    transientsDbConn = dbConns["transients"]
    cataloguesDbConn = dbConns["catalogues"]
    pmDbConn = dbConns["marshall"]
    ```
    

    .. todo ::

        - update key arguments values and definitions with defaults
        - update return values and definitions
        - update usage examples and text
        - update docstring text
        - check sublime snippet exists
        - clip any useful text to docs mindmap
        - regenerate the docs and check redendering of this docstring

    **Embed reStructuredText**

    ```eval_rst
    .. todo::

        - nice!
    ```

    **Code and Syntax Highlighting**

    Inline `code` has `back-ticks around` it.

    ```javascript
    var s = "JavaScript syntax highlighting";
    alert(s);
    ```

    ```python
    s = "Python syntax highlighting"
    print(s)
    myString = "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
    ```

    ```
    No language indicated, so no syntax highlighting. 
    But let's throw in a <b>tag</b>.
    ```

    **Mermaid**

    ```mermaid
    gantt
            dateFormat  YYYY-MM-DD
            title Adding GANTT diagram functionality to mermaid
            section A section
            Completed task            :done,    des1, 2014-01-06,2014-01-08
            Active task               :active,  des2, 2014-01-09, 3d
            Future task               :         des3, after des2, 5d
            Future task2               :         des4, after des3, 5d
            section Critical tasks
            Completed task in the critical line :crit, done, 2014-01-06,24h
            Implement parser and jison          :crit, done, after des1, 2d
            Create tests for parser             :crit, active, 3d
            Future task in critical line        :crit, 5d
            Create tests for renderer           :2d
            Add to mermaid                      :1d
    ```

    **Flowchart**

    ```flow
    s=>start: start
    e=>end: end
    o=>operation: operation
    sr=>subroutine: subroutine
    c=>condition: condition
    i=>inputoutput: inputoutput
    p=>parallel: parallel

    s->o->c
    c(yes)->i->e
    c(no)->p
    p(path1, bottom)->sr(right)->o
    p(path2, top)->o
    ```

    **Tables**

    Colons can be used to align columns.

    | Tables        | Are           | Cool  |
    | ------------- |:-------------:| -----:|
    | col 3 is      | right-aligned | $1600 |
    | col 2 is      | centered      |   $12 |
    | zebra stripes | are neat      |    $1 |

    **Definitions**

    term
    : definition

    what
        Definition lists associate a term with a definition.

    **Math**

    A formula, ${e}^{i\pi }+1=0$, inside a paragraph.

    $${e}^{i\pi }+1=0$$

    **Super/Sub Scripts**

    m^2

    x^2,y^

    x~z

    C~6~H~12~O~6

    **Citations**

    Cite a source.[p. 42][#source]

    [#source]: John Doe. *A Totally Fake Book*. Vanity Press, 2006.

    Black (2015)[#Black:2015tz]

    [#Black:2015tz]: A Bryden Black 2015, *The Lion, the Dove, & the Lamb*, Wipf and Stock Publishers

    **Task Lists**

    - [x] Completed task item
    - [ ] Unfinished task item

    **Footnote**

    Here's a sentence with a footnote[^1] in the middle of it!

    [^1]: This is the footnote.

    **Emphasis**

    Emphasis, aka italics, with *asterisks* 

    Strong emphasis, aka bold, with **asterisks**.

    Strikethrough ~~deleted~~.

    **Lists**

    1. First ordered list item
    2. Another item
        * Unordered sub-list. 
    1. Actual numbers don't matter, just that it's a number
        1. Ordered sub-list
    4. And another item.

        You can have properly indented paragraphs within list items.   

    and unordered lists:

    * Unordered list can use asterisks
    - Or minuses
    + Or pluses

    **Links**

    [I'm an inline-style link](https://www.google.com)

    **Images**

    Here's our logo (hover to see the title text):

    Inline-style: 
    ![alt text](https://github.com/adam-p/markdown-here/raw/master/src/common/images/icon48.png "Logo Title Text 1")

    Reference-style: 
    ![alt text][logo]

    [logo]: https://github.com/adam-p/markdown-here/raw/master/src/common/images/icon48.png "Logo Title Text 2"

    **Blockquotes**

    > Blockquotes are very handy in email to emulate reply text.
    > This line is part of the same quote.

    Quote break.

    > This is a very long line that will still be quoted properly when it wraps. Oh boy let's keep writing to make sure this is long enough to actually wrap for everyone. Oh, you can *put* **Markdown** into a blockquote. 

    **Abbreviations**

    The HTML specification is maintained by the W3C.

    *[HTML]: Hyper Text Markup Language
    *[W3C]:  World Wide Web Consortium

    **Horizontal Rule**

    Three or more...

    ---

    Hyphens
    """
    import os
    rootPath = os.path.dirname(__file__)

    return rootPath



#!/usr/local/bin/python
# encoding: utf-8
"""
*Convert coordinates from decimal to sexagesimal units and vice-versa*

:Author:
    David Young
"""
from __future__ import division
from builtins import str
from builtins import object
from past.utils import old_div
import sys
import os
import math
os.environ['TERM'] = 'vt100'
from fundamentals import tools

class unit_conversion(object):
    """
    *The worker class for the unit_conversion module*

    **Key Arguments**

    - ``log`` -- logger
    - ``settings`` -- the settings dictionary (prob not required)
    

    **Usage**

    .. todo::

        - add usage info
        - create a sublime snippet for usage
        - add ra_sexegesimal_to_decimal

    ```python
    usage code
    ```
    

    .. todo::

        - @review: when complete, clean unit_conversion class
        - @review: when complete add logging
        - @review: when complete, decide whether to abstract class to another module
    """
    # Initialisation
    # 1. @flagged: what are the unique attrributes for each object? Add them
    # to __init__

    def __init__(
            self,
            log,
            settings=False
    ):
        self.log = log
        log.debug("instansiating a new 'unit_conversion' object")
        self.settings = settings
        # xt-self-arg-tmpx

        # 2. @flagged: what are the default attrributes each object could have? Add them to variable attribute set here
        # Variable Data Atrributes

        # 3. @flagged: what variable attrributes need overriden in any baseclass(es) used
        # Override Variable Data Atrributes

        # Initial Actions

        return None

    # 4. @flagged: what actions does each object have to be able to perform? Add them here
    # Method Attributes
    def get(self):
        """
        *get the unit_conversion object*

        **Return**

        - ``unit_conversion``
        

        .. todo::

            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.debug('starting the ``get`` method')

        unit_conversion = None

        self.log.debug('completed the ``get`` method')
        return unit_conversion

    def dec_sexegesimal_to_decimal(
            self,
            dec):
        """
        *Convert a declination from sexegesimal format to decimal degrees.*

        Precision should be respected. If a float is passed to this method, the same float will be returned (useful if unclear which format coordinates are in).

        The code will attempt to read the sexegesimal value in whatever form it is passed. Any of the following should be handled correctly:

        - ``+1:58:05.45341``
        - ``01:5:05``
        - ``+1 58 05.45341``
        - ``-23h53m05s``

        **Key Arguments**

        - ``dec`` - DEC in sexegesimal format.
        

        **Return**

        - ``decDeg`` -- declination converted to decimal degrees
        

        **Usage**

        .. todo::

            - replace dryxPython declination_sexegesimal_to_decimal with this version in all my code
            - replace coords_sex_to_dec in all code

        ```python
        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        dec = converter.dec_sexegesimal_to_decimal(
            dec="-23:45:21.23232"
        )
        print(dec)

        # OUTPUT: -23.7558978667
        ```
        
        """
        self.log.debug(
            'completed the ````dec_sexegesimal_to_decimal`` method')

        import re

        # TEST TO SEE IF DECIMAL DEGREES PASSED
        try:
            dec = float(dec)
            if dec > -90. and dec < 90.:
                self.log.debug(
                    'declination seems to already be in decimal degrees, returning original value' % locals())
                return float(dec)
        except:
            pass

        # REMOVE SURROUNDING WHITESPACE
        dec = str(dec).strip()

        # LOOK FOR A MINUS SIGN.  NOTE THAT -00 IS THE SAME AS 00.
        regex = re.compile(
            r'^([\+\-]?(\d|[0-8]\d))\D+([0-5]\d)\D+([0-6]?\d(\.\d+)?)$')
        decMatch = regex.match(dec)

        if decMatch:
            degrees = decMatch.group(1)
            minutes = decMatch.group(3)
            seconds = decMatch.group(4)

            if degrees[0] == '-':
                sgn = -1
            else:
                sgn = 1

            degrees = abs(float(degrees))
            minutes = float(minutes)
            seconds = float(seconds)

            # PRECISION TEST
            # 1s = .000277778 DEGREE
            # THEREFORE REPORT SECONDS TO A PRECISION = INPUT PRECISION + 4
            decimalLen = len(repr(seconds).split(".")[-1])
            precision = decimalLen + 4

            decDeg = (degrees + (minutes / 60.0)
                              + (seconds / 3600.0)) * sgn

            decDeg = "%0.*f" % (precision, decDeg)

        else:
            raise IOError(
                "could not convert dec to decimal degrees, could not parse sexegesimal input. Original value was `%(dec)s`" % locals())

        decDeg = float(decDeg)
        self.log.debug('decDeg: %(decDeg)s' % locals())
        self.log.debug(
            'completed the ``dec_sexegesimal_to_decimal`` method')

        return float(decDeg)

    def ra_sexegesimal_to_decimal(
        self,
        ra
    ):
        """
        *Convert a right-ascension from sexegesimal format to decimal degrees.*

        Precision should be respected. If a float is passed to this method, the same float will be returned (useful if unclear which format coordinates are in).

        The code will attempt to read the sexegesimal value in whatever form it is passed. Any of the following should be handled correctly

        - ``23:45:21.23232``
        - ``23h45m21.23232s``
        - ``23 45 21.23232``
        - ``2 04 21.23232``
        - ``04:45  21``

        **Key Arguments**

        - ``ra`` -- ra in sexegesimal units
        

        **Return**

        - ``decimalDegrees``
        

        **Usage**

        ```python
        - replace dryxPython ra_sexegesimal_to_decimal with this version in all my code

        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        ra = converter.ra_sexegesimal_to_decimal(
            ra="04:45  21"
        )
        print(ra)

        # OUTPUT: 71.3375
        ```
        
        """
        import re

        # TEST TO SEE IF DECIMAL DEGREES PASSED
        try:
            ra = float(ra)
            if ra >= 0. and ra <= 360.:
                self.log.debug(
                    'RA seems to already be in decimal degrees, returning original value' % locals())
                return float(ra)
        except:
            pass

        # REMOVE SURROUNDING WHITESPACE
        ra = str(ra).strip()

        regex = re.compile(
            r'^(\+?(\d|[0-1]\d|2[0-3]))\D+([0-5]\d)\D+([0-6]?\d(\.\d*?)?)(s)?\s*?$')
        raMatch = regex.match(ra)

        if raMatch:
            degrees = raMatch.group(1)
            minutes = raMatch.group(3)
            seconds = raMatch.group(4)

            degrees = abs(float(degrees)) * 15.0
            minutes = float(minutes) * 15.0
            seconds = float(seconds) * 15.0

            # PRECISION TEST
            # 1s ARCSEC = .000018519 DEGREE
            # THEREFORE REPORT SECONDS TO A PRECISION = INPUT PRECISION + 5
            decimalLen = len(repr(seconds).split(".")[-1])
            precision = decimalLen + 5

            decimalDegrees = (degrees + (minutes / 60.0)
                              + (seconds / 3600.0))

            decimalDegrees = "%0.*f" % (precision, decimalDegrees)

        else:
            raise IOError(
                "could not convert ra to decimal degrees, could not parse sexegesimal input. Original value was `%(ra)s`" % locals())

        raDeg = decimalDegrees
        self.log.debug('raDeg: %(decimalDegrees)s' % locals())
        self.log.debug(
            'completed the ``ra_sexegesimal_to_decimal`` method')

        return float(raDeg)

    def ra_decimal_to_sexegesimal(
            self,
            ra,
            delimiter=":"):
        """
        *Convert a right-ascension between decimal degrees and sexegesimal.*

        Precision should be respected.

        **Key Arguments**

        - ``ra`` -- RA in decimal degrees. Will try and convert to float before performing calculation.
        - ``delimiter`` -- how to delimit the RA units. Default *:*
        

        **Return**

        - ``sexegesimal`` -- ra in sexegesimal units
        

        **Usage**

        ..  todo::

            - replace ra_to_sex from dryxPython in all code

        ```python
        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        ra = converter.ra_decimal_to_sexegesimal(
            ra="-23.454676456",
            delimiter=":"
        )
        print(ra)

        # OUT: 22:26:10.87
        ```
        
        """
        self.log.debug('starting the ``ra_decimal_to_sexegesimal`` method')

        # CONVERT RA TO FLOAT
        try:
            self.log.debug("attempting to convert RA to float")
            ra = float(ra)
        except Exception as e:
            self.log.error(
                "could not convert RA to float - failed with this error: %s " % (str(e),))
            return -1

        # COMPLAIN IF RA NOT BETWEEN -360 - 360
        if ra > 0. and ra < 360.:
            pass
        elif ra < 0 and ra > -360.:
            ra = 360. + ra
        else:
            self.log.error(
                "RA must be between 0 - 360 degrees")
            return -1

        # PRECISION TEST
        # 1s ARCSEC = .000018519 DEGREE
        # THEREFORE REPORT SECONDS TO A PRECISION = INPUT PRECISION - 5
        decimalLen = len(repr(ra).split(".")[-1])
        precision = decimalLen - 5

        # CALCULATION FROM DECIMAL DEGREES
        import math
        ra_hh = int(old_div(ra, 15))
        ra_mm = int((old_div(ra, 15) - ra_hh) * 60)
        ra_ss = int(((old_div(ra, 15) - ra_hh) * 60 - ra_mm) * 60)
        ra_ff = ((old_div(ra, 15) - ra_hh) * 60 - ra_mm) * 60 - ra_ss

        # SET PRECISION
        ra_ff = repr(ra_ff)[2:]
        ra_ff = ra_ff[:precision]
        if len(ra_ff):
            ra_ff = "." + ra_ff
        if precision < 0:
            ra_ff = ""

        sexegesimal = '%02d' % ra_hh + delimiter + '%02d' % ra_mm + \
            delimiter + '%02d' % ra_ss + ra_ff

        self.log.debug('completed the ``ra_decimal_to_sexegesimal`` method')
        return sexegesimal

    def dec_decimal_to_sexegesimal(
            self,
            dec,
            delimiter=":"):
        """
        *Convert a declination between decimal degrees and sexegesimal.*

        Precision should be respected.

        **Key Arguments**

        - ``dec`` -- DEC in decimal degrees. Will try and convert to float before performing calculation.
        - ``delimiter`` -- how to delimit the RA units. Default *:*
        

        **Return**

        - ``sexegesimal`` -- ra in sexegesimal units
        

        **Usage**

        ..  todo::

            - replace dec_to_sex in dryxPython in all code

        ```python
        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        dec = converter.dec_decimal_to_sexegesimal(
            dec="-3.454676456",
            delimiter=":"
        )
        print(dec)

        # OUT: -03:27:16.8
        ```
        
        """
        self.log.debug('starting the ``dec_decimal_to_sexegesimal`` method')

        import math

        # CONVERT DEC TO FLOAT
        try:
            self.log.debug("attempting to convert RA to float")
            dec = float(dec)
        except Exception as e:
            self.log.error(
                "could not convert RA to float - failed with this error: %s " % (str(e),))
            return -1

        # COMPLAIN IF DEC NOT BETWEEN -90 - 90
        if dec > -90. and dec < 90.:
            pass
        else:
            self.log.error(
                "DEC must be between -90 - 90 degrees")
            return -1

        if (dec >= 0):
            hemisphere = '+'
        else:
            hemisphere = '-'
            dec *= -1

        # PRECISION TEST
        # 1s = .000277778 DEGREE
        # THEREFORE REPORT SECONDS TO A PRECISION = INPUT PRECISION - 4
        decimalLen = len(repr(dec).split(".")[-1])
        precision = decimalLen - 4

        dec_deg = int(dec)
        dec_mm = int((dec - dec_deg) * 60)
        dec_ss = int(((dec - dec_deg) * 60 - dec_mm) * 60)
        dec_f = (((dec - dec_deg) * 60 - dec_mm) * 60) - dec_ss

        # SET PRECISION
        dec_f = repr(dec_f)[2:]
        dec_f = dec_f[:precision]
        if len(dec_f):
            dec_f = "." + dec_f
        if precision < 0:
            dec_f = ""

        sexegesimal = hemisphere + '%02d' % dec_deg + delimiter + \
            '%02d' % dec_mm + delimiter + '%02d' % dec_ss + dec_f

        self.log.debug('completed the ``dec_decimal_to_sexegesimal`` method')
        return sexegesimal

    # use the tab-trigger below for new method
    def ra_dec_to_cartesian(
            self,
            ra,
            dec):
        """*Convert an RA, DEC coordinate set to x, y, z cartesian coordinates*

        **Key Arguments**

        - ``ra`` -- right ascension in sexegesimal or decimal degress.
        - ``dec`` -- declination in sexegesimal or decimal degress.
        

        **Return**

        - ``cartesians`` -- tuple of (x, y, z) coordinates
        

        ..  todo::

            - replace calculate_cartesians in all code

        **Usage**

        ```python
        from astrocalc.coords import unit_conversion
        converter = unit_conversion(
            log=log
        )
        x, y, z = converter.ra_dec_to_cartesian(
            ra="23 45 21.23232",
            dec="+01:58:5.45341"
        )
        print(x, y, z)

        # OUTPUT: 0.9973699780687104, -0.06382462462791459, 0.034344492110465606
        ```
        
        """
        self.log.debug('starting the ``ra_dec_to_cartesian`` method')

        ra = self.ra_sexegesimal_to_decimal(
            ra=ra
        )
        dec = self.dec_sexegesimal_to_decimal(
            dec=dec
        )

        ra = math.radians(ra)
        dec = math.radians(dec)
        cos_dec = math.cos(dec)
        cx = math.cos(ra) * cos_dec
        cy = math.sin(ra) * cos_dec
        cz = math.sin(dec)

        cartesians = (cx, cy, cz)

        self.log.debug('completed the ``ra_dec_to_cartesian`` method')
        return cartesians

    # use the tab-trigger below for new method
    # xt-class-method



from __future__ import print_function
from builtins import str
import os
import unittest
import shutil
import unittest
import yaml
from fundamentals import tools
from astrocalc.utKit import utKit
from os.path import expanduser
home = expanduser("~")

packageDirectory = utKit("").get_project_root()
settingsFile = packageDirectory + "/test_settings.yaml"
su = tools(
    arguments={"settingsFile": settingsFile},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName=None,
    defaultSettingsFile=False
)
arguments, settings, log, dbConn = su.setup()

# SETUP PATHS TO COMMON DIRECTORIES FOR TEST DATA
moduleDirectory = os.path.dirname(__file__)
pathToInputDir = moduleDirectory + "/input/"
pathToOutputDir = moduleDirectory + "/output/"

try:
    shutil.rmtree(pathToOutputDir)
except:
    pass
# COPY INPUT TO OUTPUT DIR
shutil.copytree(pathToInputDir, pathToOutputDir)

# Recursively create missing directories
if not os.path.exists(pathToOutputDir):
    os.makedirs(pathToOutputDir)


class test_translate(unittest.TestCase):

    def test_translate_function01(self):

        from astrocalc.coords import translate
        ra, dec = translate(
            log=log,
            settings=settings,
            ra="14:45:32.3432",
            dec="-45:34:23.3434",
            northArcsec=45,
            eastArcsec=68
        ).get()

        print(ra, dec)

    def test_translate_function02(self):

        from astrocalc.coords import translate
        ra, dec = translate(
            log=log,
            settings=settings,
            ra="14.546",
            dec="-45.34232334",
            northArcsec=4560,
            eastArcsec=-5678
        ).get()

        print(ra, dec)

    def test_translate_ra_gt_360(self):

        from astrocalc.coords import translate
        ra, dec = translate(
            log=log,
            settings=settings,
            ra="14.546438",
            dec="-45.34232",
            northArcsec=4560,
            eastArcsec=+967800
        ).get()

        print(ra, dec)

    def test_translate_ra_lt_360(self):

        from astrocalc.coords import translate
        ra, dec = translate(
            log=log,
            settings=settings,
            ra="14.546438",
            dec="-45.34232334",
            northArcsec=4560,
            eastArcsec=-967800
        ).get()

        print(ra, dec)

    def test_translate_dec_lt_m90(self):

        from astrocalc.coords import translate
        ra, dec = translate(
            log=log,
            settings=settings,
            ra="14.546438",
            dec="-85.34",
            northArcsec=-43560,
            eastArcsec=-967800
        ).get()

        print(ra, dec)

    def test_translate_dec_gt_90(self):

        from astrocalc.coords import translate
        ra, dec = translate(
            log=log,
            settings=settings,
            ra="14.546438",
            dec="85.34232334",
            northArcsec=45600,
            eastArcsec=-967800
        ).get()

        print(ra, dec)

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function



from __future__ import print_function
from builtins import str
import os
import unittest
import shutil
import unittest
import yaml
from astrocalc.utKit import utKit
from fundamentals import tools
from os.path import expanduser
home = expanduser("~")

packageDirectory = utKit("").get_project_root()
settingsFile = packageDirectory + "/test_settings.yaml"
su = tools(
    arguments={"settingsFile": settingsFile},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName=None,
    defaultSettingsFile=False
)
arguments, settings, log, dbConn = su.setup()

# SETUP PATHS TO COMMON DIRECTORIES FOR TEST DATA
moduleDirectory = os.path.dirname(__file__)
pathToInputDir = moduleDirectory + "/input/"
pathToOutputDir = moduleDirectory + "/output/"

try:
    shutil.rmtree(pathToOutputDir)
except:
    pass
# COPY INPUT TO OUTPUT DIR
shutil.copytree(pathToInputDir, pathToOutputDir)

# Recursively create missing directories
if not os.path.exists(pathToOutputDir):
    os.makedirs(pathToOutputDir)


class test_separations(unittest.TestCase):

    def test_separations_function(self):
        # xt-kwarg_key_and_value

        from astrocalc.coords import separations
        calculator = separations(
            log=log,
            ra1="23:32:23.2324",
            dec1="-13:32:45.43553",
            ra2="23:32:34.642",
            dec2="-12:12:34.9334",
        )
        print(calculator.get())

        calculator = separations(
            log=log,
            ra1=2.3342343,
            dec1=89.23244233,
            ra2=45.343545345,
            dec2=87.3435435
        )
        print(calculator.get())

        calculator = separations(
            log=log,
            ra1=352.5342343,
            dec1=89.23,
            ra2="23:32:34.642",
            dec2="89:12:34.9334"
        )
        print(calculator.get())

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function
